@@ -405,12 +405,6 @@ main (int argc, char *argv[])
         gboolean              res;
         GError               *error;
 
-        manager = NULL;
-
-        if (!g_thread_supported ()) {
-                g_thread_init (NULL);
-        }
-
         gnome_settings_profile_start (NULL);
 
         bindtextdomain (GETTEXT_PACKAGE, GNOME_SETTINGS_LOCALEDIR);
