@@ -194,19 +194,29 @@ gcm_session_get_output_id (GsdColorManager *manager, GnomeRROutput *output)
                          error->message);
                 g_error_free (error);
                 g_string_append_printf (device_id,
-                                        "_%s",
+                                        "-%s",
                                         gnome_rr_output_get_name (output));
                 goto out;
         }
 
-        /* get EDID data */
+        /* check EDID data is okay to use */
         vendor = gcm_edid_get_vendor_name (edid);
+        name = gcm_edid_get_monitor_name (edid);
+        serial = gcm_edid_get_serial_number (edid);
+        if (vendor == NULL && name == NULL && serial == NULL) {
+                g_debug ("edid invalid for %s, falling back to connection name",
+                         gnome_rr_output_get_name (output));
+                g_string_append_printf (device_id,
+                                        "-%s",
+                                        gnome_rr_output_get_name (output));
+                goto out;
+        }
+
+        /* use EDID data */
         if (vendor != NULL)
                 g_string_append_printf (device_id, "-%s", vendor);
-        name = gcm_edid_get_monitor_name (edid);
         if (name != NULL)
                 g_string_append_printf (device_id, "-%s", name);
-        serial = gcm_edid_get_serial_number (edid);
         if (serial != NULL)
                 g_string_append_printf (device_id, "-%s", serial);
 out:
@@ -353,8 +363,7 @@ gcm_session_profile_assign_find_device_cb (GObject *object,
                                                res,
                                                &error);
         if (device == NULL) {
-                g_warning ("not found device %s which should have been added: %s",
-                           cd_device_get_id (device),
+                g_warning ("not found device which should have been added: %s",
                            error->message);
                 g_error_free (error);
                 gcm_session_async_helper_free (helper);
@@ -743,7 +752,7 @@ gcm_session_generate_vcgt (CdProfile *profile, guint size)
                 goto out;
 
         /* get tone curves from profile */
-        vcgt = cmsReadTag (lcms_profile, cmsSigVcgtType);
+        vcgt = cmsReadTag (lcms_profile, cmsSigVcgtTag);
         if (vcgt == NULL || vcgt[0] == NULL) {
                 g_debug ("profile does not have any VCGT data");
                 goto out;
@@ -1520,8 +1529,7 @@ gcm_session_profile_gamma_find_device_cb (GObject *object,
                                                            res,
                                                            &error);
         if (device == NULL) {
-                g_warning ("not found device %s: %s",
-                           cd_device_get_id (device),
+                g_warning ("could not find device: %s",
                            error->message);
                 g_error_free (error);
                 goto out;
