@@ -63,6 +63,7 @@
 /* Button settings */
 #define KEY_ACTION_TYPE         "action-type"
 #define KEY_CUSTOM_ACTION       "custom-action"
+#define KEY_CUSTOM_ELEVATOR_ACTION "custom-elevator-action"
 
 /* See "Wacom Pressure Threshold" */
 #define DEFAULT_PRESSURE_THRESHOLD 27
@@ -73,7 +74,7 @@ struct GsdWacomManagerPrivate
         GdkDeviceManager *device_manager;
         guint device_added_id;
         guint device_removed_id;
-        GHashTable *devices;
+        GHashTable *devices; /* key = GdkDevice, value = GsdWacomDevice */
 
         /* button capture */
         GSList *screens;
@@ -192,7 +193,6 @@ set_pressurecurve (GsdWacomDevice *device,
         gsize nvalues;
 
         property.data.i = g_variant_get_fixed_array (value, &nvalues, sizeof (gint32));
-        g_variant_unref (value);
 
         if (nvalues != 4) {
                 g_error ("Pressurecurve requires 4 values.");
@@ -200,6 +200,7 @@ set_pressurecurve (GsdWacomDevice *device,
         }
 
         wacom_set_property (device, &property);
+        g_variant_unref (value);
 }
 
 /* Area handling. Each area is defined as top x/y, bottom x/y and limits the
@@ -218,7 +219,6 @@ set_area (GsdWacomDevice  *device,
         gsize nvalues;
 
         property.data.i = g_variant_get_fixed_array (value, &nvalues, sizeof (gint32));
-        g_variant_unref (value);
 
         if (nvalues != 4) {
                 g_error ("Area configuration requires 4 values.");
@@ -226,6 +226,7 @@ set_area (GsdWacomDevice  *device,
         }
 
         wacom_set_property (device, &property);
+        g_variant_unref (value);
 }
 
 static void
@@ -278,7 +279,7 @@ set_device_buttonmap (GsdWacomDevice *device,
 
 	intmap = g_variant_get_fixed_array (value, &nmap, sizeof (gint32));
 	map = g_new0 (unsigned char, nmap);
-	for (i = 0; i < nmap && i < sizeof (map); i++)
+	for (i = 0; i < nmap; i++)
 		map[i] = intmap[i];
         g_variant_unref (value);
 
@@ -375,43 +376,88 @@ apply_stylus_settings (GsdWacomDevice *device)
 	set_pressurethreshold (device, threshold);
 }
 
-static struct {
+static void
+set_led (GsdWacomDevice *device,
+	 int             group_id,
+	 int             index)
+{
+	GError *error = NULL;
+	const char *path;
+	char *command;
+	gboolean ret;
+
+#ifndef HAVE_GUDEV
+	/* Not implemented on non-Linux systems */
+	return;
+#endif
+	g_return_if_fail (index >= 1);
+
+	path = gsd_wacom_device_get_path (device);
+
+	g_debug ("Switching group ID %d to index %d for device %s", group_id, index, path);
+
+	command = g_strdup_printf ("pkexec " LIBEXECDIR "/gsd-wacom-led-helper --path %s --group %d --led %d",
+				   path, group_id - 1, index - 1);
+	ret = g_spawn_command_line_sync (command,
+					 NULL,
+					 NULL,
+					 NULL,
+					 &error);
+
+	if (ret == FALSE) {
+		g_debug ("Failed to launch '%s': %s", command, error->message);
+		g_error_free (error);
+	}
+
+	g_free (command);
+}
+
+struct DefaultButtons {
 	const char *button;
 	int         num;
-} def_buttons[] = {
+};
+
+struct DefaultButtons def_touchrings_buttons[] = {
 	/* Touchrings */
 	{ "AbsWheelUp", 90 },
 	{ "AbsWheelDown", 91 },
+	{ "RelWheelUp", 90 },
+	{ "RelWheelDown", 91 },
 	{ "AbsWheel2Up", 92 },
 	{ "AbsWheel2Down", 93 },
+	{ NULL, 0 }
+};
+
+struct DefaultButtons def_touchstrip_buttons[] = {
 	/* Touchstrips */
 	{ "StripLeftUp", 94 },
 	{ "StripLeftDown", 95 },
 	{ "StripRightUp", 96 },
-	{ "StripRightDown", 97 }
+	{ "StripRightDown", 97 },
+	{ NULL, 0 }
 };
-#define NUM_TOUCH_BUTTONS 4
 
 static void
-reset_touch_buttons (XDevice    *xdev,
-		     guint       offset,
-		     const char *device_property)
+reset_touch_buttons (XDevice               *xdev,
+		     struct DefaultButtons *buttons,
+		     const char            *device_property)
 {
-	Atom actions[NUM_TOUCH_BUTTONS];
+	Atom actions[6];
 	Atom action_prop;
 	guint i;
 
 	/* Create a device property with the action for button i */
-	for (i = 0; i < NUM_TOUCH_BUTTONS; i++)
+	for (i = 0; buttons[i].button != NULL; i++)
 	{
 		char *propname;
 		int action[2]; /* press + release */
 		Atom prop;
-		int mapped_button = def_buttons[i + offset].num;
+		int mapped_button = buttons[i].num;
 
-		action[0] = AC_BUTTON | mapped_button;
+		action[0] = AC_BUTTON | AC_KEYBTNPRESS | mapped_button;
+		action[1] = AC_BUTTON | mapped_button;
 
-		propname = g_strdup_printf ("Button %s action", def_buttons[i + offset].button);
+		propname = g_strdup_printf ("Button %s action", buttons[i].button);
 		prop = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), propname, False);
 		g_free (propname);
 		XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev,
@@ -427,7 +473,7 @@ reset_touch_buttons (XDevice    *xdev,
 	action_prop = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device_property, True);
 	XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev,
 			       action_prop, XA_ATOM, 32, PropModeReplace,
-			       (const guchar *) actions, NUM_TOUCH_BUTTONS);
+			       (const guchar *) actions, i);
 }
 
 static void
@@ -463,13 +509,16 @@ reset_pad_buttons (GsdWacomDevice *device)
 	g_free (map);
 
 	gdk_error_trap_push ();
-	reset_touch_buttons (xdev, 0, "Wacom Wheel Buttons");
-	reset_touch_buttons (xdev, 4, "Wacom Strip Buttons");
+	reset_touch_buttons (xdev, def_touchrings_buttons, "Wacom Wheel Buttons");
+	reset_touch_buttons (xdev, def_touchstrip_buttons, "Wacom Strip Buttons");
 	gdk_error_trap_pop_ignored ();
 
 	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
 
-	/* FIXME, set the LED(s) for the mode(s) too */
+	/* Reset all the LEDs */
+	/* FIXME, get the number of modes somewhere else */
+	for (i = 1; i <= 4; i++)
+		set_led (device, i, 1);
 }
 
 static void
@@ -489,6 +538,11 @@ set_wacom_settings (GsdWacomManager *manager,
 
         type = gsd_wacom_device_get_device_type (device);
 
+	if (type == WACOM_TYPE_TOUCH) {
+		set_absolute (device, FALSE);
+		return;
+	}
+
 	if (type == WACOM_TYPE_CURSOR) {
 		GVariant *values[4], *variant;
 		guint i;
@@ -543,7 +597,8 @@ wacom_settings_changed (GSettings      *settings,
 		set_tpcbutton (device, g_settings_get_boolean (settings, key));
 	} else if (g_str_equal (key, KEY_IS_ABSOLUTE)) {
 		if (type != WACOM_TYPE_CURSOR &&
-		    type != WACOM_TYPE_PAD)
+		    type != WACOM_TYPE_PAD &&
+		    type != WACOM_TYPE_TOUCH)
 			set_absolute (device, g_settings_get_boolean (settings, key));
 	} else if (g_str_equal (key, KEY_AREA)) {
 		if (type != WACOM_TYPE_CURSOR &&
@@ -691,23 +746,117 @@ struct {
 
 static void
 send_modifiers (Display *display,
-		XDevice *dev,
 		guint mask,
 		gboolean is_press)
 {
 	guint i;
 
+	if (mask == 0)
+		return;
+
 	for (i = 0; i < G_N_ELEMENTS(mods_keysyms); i++) {
 		if (mask & mods_keysyms[i].mask) {
 			guint keycode;
 
 			keycode = XKeysymToKeycode (display, mods_keysyms[i].keysym);
-			XTestFakeDeviceKeyEvent (display, dev, keycode,
-						 is_press ? True : False, NULL, 0, 0);
+			XTestFakeKeyEvent (display, keycode,
+					   is_press ? True : False, 0);
 		}
 	}
 }
 
+static char *
+get_elevator_shortcut_string (GSettings        *settings,
+			      GtkDirectionType  dir)
+{
+	char **strv, *str;
+
+	strv = g_settings_get_strv (settings, KEY_CUSTOM_ELEVATOR_ACTION);
+	if (strv == NULL)
+		return NULL;
+
+	if (g_strv_length (strv) >= 1 && dir == GTK_DIR_UP)
+		str = g_strdup (strv[0]);
+	else if (g_strv_length (strv) >= 2 && dir == GTK_DIR_DOWN)
+		str = g_strdup (strv[1]);
+	else
+		str = NULL;
+
+	g_strfreev (strv);
+
+	return str;
+}
+
+static void
+generate_key (GsdWacomTabletButton *wbutton,
+	      int                   group,
+	      Display              *display,
+	      GtkDirectionType      dir,
+	      gboolean              is_press)
+{
+	char                 *str;
+	guint                 keyval;
+	guint                *keycodes;
+	guint                 keycode;
+	guint                 mods;
+	GdkKeymapKey         *keys;
+	int                   n_keys;
+	guint                 i;
+
+	if (wbutton->type == WACOM_TABLET_BUTTON_TYPE_ELEVATOR)
+		str = get_elevator_shortcut_string (wbutton->settings, dir);
+	else
+		str = g_settings_get_string (wbutton->settings, KEY_CUSTOM_ACTION);
+
+	if (str == NULL)
+		return;
+
+	gtk_accelerator_parse_with_keycode (str, &keyval, &keycodes, &mods);
+	if (keycodes == NULL) {
+		g_warning ("Failed to find a keycode for shortcut '%s'", str);
+		g_free (str);
+		return;
+	}
+	g_free (keycodes);
+
+	/* Now look for our own keycode, in the group as us */
+	if (!gdk_keymap_get_entries_for_keyval (gdk_keymap_get_default (), keyval, &keys, &n_keys)) {
+		g_warning ("Failed to find a keycode for keyval '%s' (0x%x)", gdk_keyval_name (keyval), keyval);
+		g_free (str);
+		return;
+	}
+
+	keycode = 0;
+	for (i = 0; i < n_keys; i++) {
+		if (keys[i].group != group)
+			continue;
+		if (keys[i].level > 0)
+			continue;
+		keycode = keys[i].keycode;
+	}
+	/* Couldn't find it in the current group? Look in group 0 */
+	if (keycode == 0) {
+		for (i = 0; i < n_keys; i++) {
+			if (keys[i].level > 0)
+				continue;
+			keycode = keys[i].keycode;
+		}
+	}
+	g_free (keys);
+
+	g_debug ("Emitting '%s' (keyval: %d, keycode: %d mods: 0x%x)",
+		 str, keyval, keycode, mods);
+	g_free (str);
+
+	/* And send out the keys! */
+	if (is_press)
+		send_modifiers (display, mods, TRUE);
+	XTestFakeKeyEvent (display, keycode,
+			   is_press ? True : False, 0);
+	if (is_press == FALSE)
+		send_modifiers (display, mods, FALSE);
+}
+
 static GdkFilterReturn
 filter_button_events (XEvent          *xevent,
                       GdkEvent        *event,
@@ -721,11 +870,6 @@ filter_button_events (XEvent          *xevent,
 	int                  button;
 	GsdWacomTabletButton *wbutton;
 	GtkDirectionType      dir;
-	XDevice               dev;
-	char                 *str;
-	guint                 keyval;
-	guint                *keycodes;
-	guint                 mods;
 
         /* verify we have a key event */
 	if (xevent->type != GenericEvent)
@@ -749,7 +893,6 @@ filter_button_events (XEvent          *xevent,
 
 	button = xev->detail;
 
-	/* FIXME, we'll also need to pass the current mode(s) */
 	wbutton = gsd_wacom_device_get_button (device, button, &dir);
 	if (wbutton == NULL) {
 		g_warning ("Could not find matching button for '%d' on '%s'",
@@ -757,7 +900,8 @@ filter_button_events (XEvent          *xevent,
 		return GDK_FILTER_CONTINUE;
 	}
 
-	g_debug ("Received event button '%s'%s ('%d') on device '%s' ('%d')",
+	g_debug ("Received event button %s '%s'%s ('%d') on device '%s' ('%d')",
+		 xiev->evtype == XI_ButtonPress ? "press" : "release",
 		 wbutton->id,
 		 wbutton->type == WACOM_TABLET_BUTTON_TYPE_ELEVATOR ?
 		 (dir == GTK_DIR_UP ? " 'up'" : " 'down'") : "",
@@ -765,39 +909,24 @@ filter_button_events (XEvent          *xevent,
 		 gsd_wacom_device_get_name (device),
 		 deviceid);
 
-	/* FIXME, don't know how to handle those yet */
-	if (wbutton->type == WACOM_TABLET_BUTTON_TYPE_ELEVATOR)
-		return GDK_FILTER_REMOVE;
+	if (wbutton->type == WACOM_TABLET_BUTTON_TYPE_HARDCODED) {
+		int new_mode;
 
-	/* FIXME, we need to switch mode here */
-	if (wbutton->type == WACOM_TABLET_BUTTON_TYPE_HARDCODED)
+		/* We switch modes on key release */
+		if (xiev->evtype == XI_ButtonRelease)
+			return GDK_FILTER_REMOVE;
+
+		new_mode = gsd_wacom_device_set_next_mode (device, wbutton->group_id);
+		set_led (device, wbutton->group_id, new_mode);
 		return GDK_FILTER_REMOVE;
+	}
 
 	/* Nothing to do */
 	if (g_settings_get_enum (wbutton->settings, KEY_ACTION_TYPE) == GSD_WACOM_ACTION_TYPE_NONE)
 		return GDK_FILTER_REMOVE;
 
-	str = g_settings_get_string (wbutton->settings, KEY_CUSTOM_ACTION);
-	gtk_accelerator_parse_with_keycode (str, &keyval, &keycodes, &mods);
-
-	if (keycodes == NULL) {
-		g_warning ("Failed to find a keycode for shortcut '%s'", str);
-		g_free (str);
-		return GDK_FILTER_REMOVE;
-	}
-	g_free (str);
-
-	dev.device_id = deviceid; /* that's cheating, but whot did it first */
-
-	/* And send out the keys! */
-	send_modifiers (xev->display, &dev, mods, TRUE);
-	XTestFakeDeviceKeyEvent (xev->display, &dev, keycodes[0],
-				 True, NULL, 0, 0);
-	XTestFakeDeviceKeyEvent (xev->display, &dev, keycodes[0],
-				 False, NULL, 0, 0);
-	send_modifiers (xev->display, &dev, mods, FALSE);
-
-	g_free (keycodes);
+	/* Send a key combination out */
+	generate_key (wbutton, xev->group.effective, xev->display, dir, xiev->evtype == XI_ButtonPress ? True : False);
 
 	return GDK_FILTER_REMOVE;
 }
