@@ -39,6 +39,7 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <libupower-glib/upower.h>
+#include <X11/Xatom.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 
@@ -1525,6 +1526,7 @@ rotate_touchscreens (GsdXrandrManager *mgr,
                                 .name = "Evdev Axes Swap",
                                 .nitems = 1,
                                 .format = 8,
+                                .type   = XA_INTEGER,
                                 .data.c = &c,
                         };
 
@@ -1544,6 +1546,7 @@ rotate_touchscreens (GsdXrandrManager *mgr,
                                         .name = "Evdev Axis Inversion",
                                         .nitems = 2,
                                         .format = 8,
+                                        .type   = XA_INTEGER,
                                         .data.c = axis,
                                 };
 
